﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

namespace BxRestClient
{
    /** This JAX-RX c# client class shows the available REST functions within BaseX. */
    public class JaxRxClient
    {
        private const string query="/books/book";
        private const string jaxrx = "http://localhost:8984/basex/jax-rx";
        private const string qct = "application/query+xml";
        private const string rm = "Response code: ";
        private const string resource = "/example";

        /** An HTTP GET request on the JAX-RX root /basex/jax-rx responses a list of availble resources. */
        public void GetAll()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx);
            var res = (HttpWebResponse)req.GetResponse();
            PrintToConsole(res.GetResponseStream());
            Console.WriteLine();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
        }

        /** An HTTP GET request on a special resource e.g., /basex/jax-rx/{resource} responses with the resource. */
        public void GetResource()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx + resource);
            var res = (HttpWebResponse)req.GetResponse();
            PrintToConsole(res.GetResponseStream());
            Console.WriteLine();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
        }

        /** XQuery execution can be performed on XML resource via GET and POST. In GET requests an HTTP parameter 'query' contains the XQuery or XPath to be evaluated. */
        public void QueryByGet()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource+"?query="+query);
            var res = (HttpWebResponse)req.GetResponse();
            var input = res.GetResponseStream();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
            if (code == HttpStatusCode.OK)
                PrintToConsole(res.GetResponseStream());
        }

        /** XQuery / XPath expressions can be performed via HTTP POST, where the query must be in the body of the request. */
        public void QueryByPost()
        {
            var container = "<query><text>" + query + "</text></query>";
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource);
            req.Method = WebRequestMethods.Http.Post;
            req.ContentType = qct;
            WriteToOutput(req.GetRequestStream(), container);
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
            if (code == HttpStatusCode.OK)
                PrintToConsole(res.GetResponseStream());
            
        }

        /** With HTTP PUT it is possible to create new resources or to update (override) existing resources. */
        public void Update()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource);
            req.Method = WebRequestMethods.Http.Put;
            req.ContentType = "text/xml";
            WriteToOutput(req.GetRequestStream(), "<books><book>Visual C# 2010</book></books>");
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
        }

        /** With HTTP POST it is also possible to add new XML documents to the existing database. */
        public void Add()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource);
            req.ContentType = "application/xml";
            req.Method = WebRequestMethods.Http.Post;
            WriteToOutput(req.GetRequestStream(), "<books><book>Another Book</book></books>");
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
            if (code == HttpStatusCode.Created)
                PrintToConsole(res.GetResponseStream());
        }

        /** With HTTP DELETE you can delete an existing resource. */
        public void Delete()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource);
            req.Method = "DELETE";
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
        }

        /** Via HTTP POST it is also possible to send commands to basex. (It is also possible to execute commands via HTTP GET as sending queries above in #QueryByGet.)*/
        public void CommandByPost()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx+resource);
            req.ContentType = qct;
            req.Method = WebRequestMethods.Http.Post;
            WriteToOutput(req.GetRequestStream(),"<command><text>info</text></command>");
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
            if (code == HttpStatusCode.OK)
                PrintToConsole(res.GetResponseStream());
        }

        /** Via HTTP POST it is also possible to execute existing XQuery skripts, which are located on the web server. (It is also possible to run XQuery skripts via HTTP GET as sending queries above in #QueryByGet)*/
        public void RunByPost()
        {
            var req = (HttpWebRequest)WebRequest.Create(jaxrx);
            req.ContentType = qct;
            req.Method = WebRequestMethods.Http.Post;
            WriteToOutput(req.GetRequestStream(), "<run><text>getnames.xq</text><variable name='position' value='2'/></run>");
            var res = (HttpWebResponse)req.GetResponse();
            var code = res.StatusCode;
            Console.WriteLine(rm + code);
            if (code == HttpStatusCode.OK)
                PrintToConsole(res.GetResponseStream());
        }

        /** This function prints the input from the HTTP response to the console. */
        private void PrintToConsole(Stream input)
        {
            int ch;
            while ((ch = input.ReadByte()) != -1)
                Console.Write((char)ch);
        }

        /** This function writes a string to the request stream. */
        private void WriteToOutput(Stream output, string content)
        {     
            var wr = new StreamWriter(output);
            wr.Write(content);
            wr.Close();
        }

    }
}
